package com.caimei365.order.controller;

import com.caimei365.order.model.ResponseJson;
import com.caimei365.order.model.dto.ReceiptAuthDto;
import com.caimei365.order.model.dto.ReceiptDto;
import com.caimei365.order.model.po.ReceiptPo;
import com.caimei365.order.model.vo.*;
import com.caimei365.order.service.ReceiptService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 线下收款接口
 *
 * @author : Charles
 * @date : 2022/2/11
 */
@Api(tags = "线下收款API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order/receipt")
public class ReceiptApi {
    private final ReceiptService receiptService;

    /**
     * 线下收款用户微信授权
     *
     * @param receiptAuthDto {
     *                       code          微信授权code
     *                       encryptedData 微信加密数据
     *                       iv            加密算法的初始向量
     *                       }
     */
    @ApiOperation("线下收款用户微信授权(小程序)")
    @PostMapping("/auth")
    public ResponseJson<ReceiptUserVo> receiptAuthorization(ReceiptAuthDto receiptAuthDto, @RequestHeader HttpHeaders headers) {
        String code = receiptAuthDto.getCode();
        String encryptedData = receiptAuthDto.getEncryptedData();
        String iv = receiptAuthDto.getIv();
        if (!StringUtils.hasLength(code)) {
            return ResponseJson.error("没有获取到微信授权code", null);
        }
        return receiptService.receiptAuthorization(code, encryptedData, iv, headers);
    }

    /**
     * 线下收款用户密码登录
     *
     * @param receiptAuthDto {
     *                       code          微信授权code
     *                       encryptedData 微信加密数据
     *                       iv            加密算法的初始向量
     *                       mobile        手机号
     *                       password      密码
     *                       }
     */
    @ApiOperation("线下收款用户密码登录")
    @PostMapping("/password")
    public ResponseJson<ReceiptUserVo> passwordAuthorization(ReceiptAuthDto receiptAuthDto, @RequestHeader HttpHeaders headers) {
        String mobile = receiptAuthDto.getMobile();
        String password = receiptAuthDto.getPassword();
        String code = receiptAuthDto.getCode();
        String encryptedData = receiptAuthDto.getEncryptedData();
        String iv = receiptAuthDto.getIv();
        if (!StringUtils.hasLength(code)) {
            return ResponseJson.error("没有获取到微信授权code", null);
        }
        if (StringUtils.hasLength(mobile) && StringUtils.hasLength(password)) {
            return receiptService.passwordAuthorization(mobile, password, code, encryptedData, iv, headers);
        } else {
            return ResponseJson.error("手机号或密码不能为空", null);
        }
    }

    /**
     * 获取收款类型
     */
    @ApiOperation("获取收款类型")
    @GetMapping("/type")
    public ResponseJson<List<ReceiptTypeVo>> getReceiptType() {
        return receiptService.getReceiptType();
    }

    /**
     * 识别收款短信
     *
     * @param receiptDto {
     *                   smsContent          短信类容
     *                   }
     */
    @ApiOperation("识别收款短信")
    @PostMapping("/read/sms")
    public ResponseJson<ReceiptVo> readSmsMessage(ReceiptDto receiptDto) {
        String smsContent = receiptDto.getSmsContent();
        if (StringUtils.hasLength(smsContent)) {
            return receiptService.readSmsMessage(smsContent, receiptDto.getOpenid());
        } else {
            return ResponseJson.error("收款短信不能为空", null);
        }
    }

    /**
     * 保存收款信息
     */
    @ApiOperation("保存收款信息")
    @PostMapping("/save")
    public ResponseJson<ReceiptPo> saveReceipt(ReceiptDto receiptDto) {
        return receiptService.saveReceipt(receiptDto);
    }

    /**
     * 获取收款列表
     */
    @ApiOperation("获取收款列表")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "receiptType", value = "收款款项类型：1订单款，2非订单款，3返佣款 4订单款或者非订单款（因财务阶段无法区分订单非订单）, 5供应商退款"),
            @ApiImplicitParam(required = false, name = "receiptStatus", value = "收款状态：1待确认、2已确认(待审核)、3审核通过、4审核未通过、5收款撤销【线上支付成功为审核通过】"),
            @ApiImplicitParam(required = false, name = "smsContent", value = "收款短信"),
            @ApiImplicitParam(required = false, name = "startDate", value = "筛选开始时间"),
            @ApiImplicitParam(required = false, name = "endDate", value = "筛选结束时间"),
            @ApiImplicitParam(required = false, name = "openid", value = "微信openid"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ReceiptVo>> getReceiptList(String receiptType,
                                                            String receiptStatus,
                                                            String smsContent,
                                                            String startDate,
                                                            String endDate,
                                                            String openid,
                                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (!StringUtils.hasLength(openid)) {
            return ResponseJson.error("微信openid不能为空！", null);
        }
        return receiptService.getReceiptList(receiptType, receiptStatus, smsContent, startDate, endDate, openid, pageNum, pageSize);
    }

    /**
     * 获取收款详情
     */
    @ApiOperation("获取收款详情")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "id", value = "款项Id"),
            @ApiImplicitParam(required = false, name = "openid", value = "微信openid")
    })
    @GetMapping("/list")
    public ResponseJson<ReceiptVo> getReceiptDetail(Integer id, String openid) {
        if (!StringUtils.hasLength(openid)) {
            return ResponseJson.error("微信openid不能为空！", null);
        }
        return receiptService.getReceiptDetail(id, openid);
    }

    /**
     * 获取收款订单列表
     */
    @ApiOperation("获取收款订单列表")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "id", value = "收款ID"),
            @ApiImplicitParam(required = false, name = "type", value = "类型：0商品订单(默认)，1订金订单"),
            @ApiImplicitParam(required = false, name = "organizeId", value = "组织ID"),
            @ApiImplicitParam(required = false, name = "orderReceiptStatus", value = "订单收款状态：1待收款，2部分收款，3已收款"),
            @ApiImplicitParam(required = false, name = "userName", value = "客户名称"),
            @ApiImplicitParam(required = false, name = "openid", value = "微信openid"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<OrderVo>> getReceiptOrderList(Integer id,
                                                               Integer type,
                                                               Integer organizeId,
                                                               String orderReceiptStatus,
                                                               String userName,
                                                               String openid,
                                                               @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (!StringUtils.hasLength(openid)) {
            return ResponseJson.error("微信openid不能为空！", null);
        }
        return receiptService.getReceiptOrderList(id, type, organizeId, orderReceiptStatus, userName, openid, pageNum, pageSize);
    }

    /**
     * 供应商退款关联的子订单信息
     */
    @ApiOperation("供应商退款关联的子订单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "id", value = "返佣收款ID"),
            @ApiImplicitParam(required = false, name = "confirmedType", value = "0待确认，2已确认子订单"),
            @ApiImplicitParam(required = false, name = "shopName", value = "供应商名称"),
            @ApiImplicitParam(required = false, name = "userName", value = "客户名称"),
            @ApiImplicitParam(required = false, name = "openid", value = "微信openid"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<ShopOrderVo>> getRefundShopOrderList(Integer id,
                                                                      Integer confirmedType,
                                                                      String shopName,
                                                                      String userName,
                                                                      String openid,
                                                                      @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                      @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (!StringUtils.hasLength(openid)) {
            return ResponseJson.error("微信openid不能为空！", null);
        }
        return receiptService.getRefundShopOrderList(id, confirmedType, shopName, userName, openid, pageNum, pageSize);
    }

}

